@@ -10,9 +10,10 @@ from email.MIMEText import MIMEText
 from email.MIMEAudio import MIMEAudio
 from email.MIMEBase import MIMEBase
 from email.MIMEImage import MIMEImage
-from Acquisition import aq_parent
+from Acquisition import aq_inner,aq_parent
 
 from Products.CMFCore.utils import getToolByName
+from Products.CMFPlone.utils import safe_unicode
 from Products.Archetypes.public import (
 #    ATFieldProperty,
     DisplayList,
@@ -38,6 +39,7 @@ from Products.PloneFormGen.content.actionAdapter import (
     TextField,
     BooleanWidget,
     IntegerWidget,
+    LinesWidget,
     MultiSelectionWidget,
     RichWidget,
     SelectionWidget,
@@ -46,6 +48,8 @@ from Products.PloneFormGen.content.actionAdapter import (
 )
 from Products.PloneFormGen.content.formMailerAdapter import formMailerAdapterSchema, FormMailerAdapter
 from Products.PloneFormGen import HAS_PLONE25, HAS_PLONE30
+#from Products.PloneFormGen.interfaces import IPloneFormGenField
+from Products.PloneFormGen.content.fieldsBase import BaseFormField
 
 from Products.PFGVerkkomaksut import PFGVerkkomaksutMessageFactory as _
 from Products.PFGVerkkomaksut.config import PROJECTNAME
@@ -150,6 +154,21 @@ PFGVerkkomaksutSchema = ATFolderSchema.copy() + formMailerAdapterSchema.copy() +
         enforceVocabulary=True,
     ),
 
+    LinesField(
+        name='order_description',
+        schemata='verkkomaksut',
+        required=False,
+        searchable=False,
+        languageIndependent=True,
+        storage=AnnotationStorage(),
+        widget=LinesWidget(
+            label=_(u'Order Description'),
+            description=_(u'Please line up the ID of the field to be added to the ORDER_DESCRIPTION for verkkomaksut.'),
+            cols=20,
+        ),
+        default_method='field_ids',
+    ),
+
     TextField(
         name='thanks_text',
         schemata='verkkomaksut',
@@ -229,7 +248,6 @@ class PFGVerkkomaksut(ATFolder, FormMailerAdapter):
         self.setNext_order_number(order_number + 1)
         REQUEST.set('ORDER_NUMBER', ORDER_NUMBER)
         if self.getMsg_necessary():
-#            super(PFGVerkkomaksut, self).onSuccess(fields, REQUEST)
             self.send_form(fields, REQUEST)
         field = self.selected_price_field()
         form = self.REQUEST.form
@@ -240,18 +258,18 @@ class PFGVerkkomaksut(ATFolder, FormMailerAdapter):
         ORDER_NUMBER = str(order_number)
         # Increase next_order_number by one.
         self.setNext_order_number(order_number + 1)
-        ORDER_DESCRIPTION = 'Payment from PloneFormGen Verkkomaksut Adapter'
+        ORDER_DESCRIPTION = self.order_description(fields, REQUEST)
         CURRENCY = 'EUR'
 #        portal = getToolByName(self, 'portal_url').getPortalObject()
 #        portal_url = portal.absolute_url()
-        parent = aq_parent(self)
+        parent = aq_parent(aq_inner(self))
 #        ids = parent.objectIds()
 #        ids = [id for id in ids if id in form.keys()]
 #        queries = ['&'+id+'='+form.get(id) for id in ids if form.get(id) is not None][1:]
 #        query = '?'+ids[0]+'='+form.get(ids[0])+''.join(queries)
         parent_url = parent.absolute_url()
         return_url = '%s/verkkomaksut-thanks' % (self.absolute_url())
-        cancel_url = '%s?portal_status_message=You have canceled'% parent_url
+        cancel_url = '%s?portal_status_message=%s'%(parent_url, self.getCancel_message())
         notify_url = '%s/verkkomaksutnotify' % (self.absolute_url())
 #        notify_url = '%s/verkkomaksutnotify%s' % (self.absolute_url(), query)
         RETURN_ADDRESS = return_url
@@ -315,6 +333,15 @@ class PFGVerkkomaksut(ATFolder, FormMailerAdapter):
         url = '%s/verkkomaksut' % parent_url
         return self.REQUEST.RESPONSE.redirect(url)
 
+    def order_description(self, fields, REQUEST=None):
+        form = self.REQUEST.form
+        values = [safe_unicode(form.get(key, '')) for key in self.getOrder_description()]
+        res = u','.join(values)
+        portal = getToolByName(self, 'portal_url').getPortalObject()
+        charset = portal.getProperty('email_charset', 'utf-8')
+        return res[:65000].encode(charset)
+
+
     def send_form(self, fields, request, **kwargs):
         """Send the form.
         """
@@ -432,5 +459,11 @@ class PFGVerkkomaksut(ATFolder, FormMailerAdapter):
         obj = reference_catalog.lookupObject(uid)
         return obj.getId()
 
+    def field_ids(self):
+        catalog = getToolByName(self, 'portal_catalog')
+        parent = aq_parent(aq_inner(self))
+        ids = [obj.id for obj in parent.contentValues() if isinstance(obj, BaseFormField)]
+        return ids
+
 #registerType(PFGVerkkomaksut)
 registerATCT(PFGVerkkomaksut, PROJECTNAME)
